c-------------------------------------------------------------------------------
c Generate the BEM mesh (using constant elements) for a square plate with
c many evenly or randomly distributed holes for the 2-D thermal analysis
c
c By Yijun Liu                                                   July 15, 2004
c                                              Revised for CHBIE: June 3, 2007
c-------------------------------------------------------------------------------

	implicit double precision (a-h,o-z)

	parameter (n0=8000000)
	dimension y(2,n0), ind(2,n0), bc(2,n0), yhole(2,1000)
	character*9 Today

      open(6, file='input.dat', status='unknown')
      open(7, file='input.plt', status='unknown')

	write(*,*) 'Give a, b, mplate for the square plate:'
	read(*,*) a, b, mplate
c      a = 1.D0
c      b = 1.D0
c      mplate = 400
	write(*,*) 'Give r, mhole, nx, ny, and nrandom for the holes:'
	read(*,*) r, mhole, nx, ny, nrandom
	write(*,*) 'Give ratio of a/b:'
	read(*,*) ratio

c Square plate:

	mplate4 = mplate/4
	dx = a/mplate4
	dy = b/mplate4
	do i=1,mplate4
	  y(1,i          ) = dx*(i-1)
	  y(2,i          ) = 0.
	  y(1,i+mplate4  ) = a
	  y(2,i+mplate4  ) = dy*(i-1)
	  y(1,i+mplate4*2) = a - dx*(i-1)
	  y(2,i+mplate4*2) = b
	  y(1,i+mplate4*3) = 0.
	  y(2,i+mplate4*3) = b - dy*(i-1)
	enddo
	do m=1,mplate
	  ind(1,m) = m
	  ind(2,m) = m + 1
	enddo
	  ind(2,mplate) = 1

c Holes:

	do i=1,mhole
	 dtheta = 360./float(mhole)
	 yhole(1,i) = r*cosd((i-1)*dtheta) * ratio
	 yhole(2,i) = r*sind((i-1)*dtheta)
	enddo

	numhole = nx*ny
	cx = a/nx
	cy = b/ny
	iseedx = 123456789
	iseedy = 987654321

	if(nrandom .eq. 0) then

c Uniform disribution:

	  do iy=1,ny
	  do ix=1,nx
	  do ih=1,mhole
	    i = mplate + mhole*(ny*(iy-1)+(ix-1)) + ih
	    y(1,i) = yhole(1,ih) + cx*(ix-0.5)
	    y(2,i) = yhole(2,ih) + cy*(iy-0.5)
	    ind(1,i) = i+1
	    ind(2,i) = i
	  enddo
		ind(1,i) = mplate + mhole*(ny*(iy-1)+(ix-1)) + 1
	  enddo
	  enddo
	
	else if(nrandom .eq. 1) then

c Random distribution:

	  r = r*1.05
	  do ihole=1,numhole
10      continue
	    cx = ran(iseedx)*(a-2*r) + r
	    cy = ran(iseedy)*(b-2*r) + r
	    itag = 0
	    do i=mplate+1, mplate + mhole*(ihole-1)
	      dist = sqrt((y(1,i)-cx)**2 + (y(2,i)-cy)**2)
	      if(dist .le. r) itag = itag + 1
	    enddo
	    if(itag .eq. 0) then
	      do ih=1,mhole
	        i = mplate + mhole*(ihole-1) + ih
	        y(1,i) = yhole(1,ih) + cx
	        y(2,i) = yhole(2,ih) + cy
	        ind(1,i) = i+1
	        ind(2,i) = i
	      enddo
	    	ind(1,i) = mplate + mhole*(ihole-1) + 1
          else 
		  goto 10
	    endif
	  enddo

	endif

c Total number of elements/nodes:

	n = mplate + numhole*mhole
	m = n
	nfield = 0

c Write the output:

	call date(Today)
	write(6,100) Today
100   format(" A Square Plate with Holes under Thermal Load", 18x, A9)
      write(6,200) m, nfield
200   format(2I8, 12x, " ! No. of Elements, No. of Interior Points")
      write(6,*) "   1.D0    1.D0             ! alpha, beta"

	write(6,*) '# Nodes:'
	do i=1,n
	write(6,300) i, y(1,i), y(2,i)
300   format(I10, 2E25.16)
	enddo

	write(6,*) '# Elements and Boundary Conditions:'

	dTemp = 0.
	do iedge=1,4
	do im=1,mplate4
	  i = mplate4*(iedge-1) + im
	  if(mod(iedge,2).eq.0) then
	    write(6,400) i, ind(1,i), ind(2,i), y(1,i)
	  else
	    write(6,500) i, ind(1,i), ind(2,i), dTemp
	  endif
	enddo
	enddo
	do i=mplate+1,n
	write(6,500) i, ind(1,i), ind(2,i), dTemp
	enddo
400   format(I10, 5x, 2I10, 15x, '1.0', E18.8)
500   format(I10, 5x, 2I10, 15x, '2.0', E18.8)
	write(6,*) '#End of File'

c Write the data file for plotting (Tecplot format):

	write(7,*) 'TITLE = "A Square Plate with Holes" '
	write(7,*) 'ZONES" '
	write(7,*) 'VARIABLES = "X", "Y" '
	write(7,*) 'ZONE T="Plate", N=',n, ', E=',m,
     &           'DATAPACKING=POINT, ZONETYPE=FELINESEG'

	write(7,600) (y(1,i),y(2,i), i=1,n)
600   format(2E20.8)
	write(7,*)
	write(7,700) (ind(1,i),ind(2,i), i=1,n)
700   format(2I20)

      call CPU_Time(time)
	write(*,*)
      write(*,*) ' Total CPU time used =', time, '(sec)'

	stop
	end